/***************************************************************************
                          ksubeditor.cpp  -  description
                             -------------------
    begin                : pi sty 10 21:05:38 CET 2003
    copyright            : (C) 2003 by Anatol Ogrek
    email                : mroova@linux.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#define READ_ERROR    -2
#define SUB_INVALID   -1
#define SUB_MICRODVD  0
#define SUB_SUBRIP    1
#define SUB_SUBVIEWER 2
#define SUB_SAMI      3
#define SUB_VPLAYER   4
#define SUB_RT        5
#define SUB_SSA       6
#define SUB_DUNNOWHAT 7		// FIXME what format is it ?
#define SUB_MPSUB     8
#define SUB_AQTITLE   9
#define SUB_MAX_TEXT  3
#define LINE_LEN  1000
#define ID_PLAY 14000
#define ID_MOVE_TIME 14010
#define ID_CHARSET 14020


 
// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qvbox.h>
#include <qwhatsthis.h>
#include <qtooltip.h>
#include <qtoolbutton.h>
#include <qstring.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qtabdialog.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <kpopupmenu.h>
#include <kio/netaccess.h>
#include <kkeydialog.h>
#include <kspell.h>

// application specific includes
#include "ksubeditor.h"
#include "ksubeditorview.h"
#include "ksubeditordoc.h"

//other includes
#include "microdvd.h"
#include "spellconf.h"
#include <stdlib.h>
KsubeditorApp::KsubeditorApp():KMainWindow(0,"Ksubeditor")
{

  KsubeditorConfig = new configuration();
  configDialog = new configure(this,"Config",true);

  //setting encoding
  int n = KsubeditorConfig->codecList->count();

  configDialog->encoding->clear();
  QString local = "Local (";
  local += QTextCodec::codecForLocale()->name();
  local += ")";
  configDialog->encoding->insertItem(local);
  configDialog->encoding->insertItem("Unicode");
  configDialog->encoding->insertItem("Latin");
  configDialog->encoding->insertItem("Microsoft Unicode");

  int i;

  for ( i = 4; i < n; i++ )
      configDialog->encoding->insertItem( KsubeditorConfig->codecList->at(i-4)->name());

///end of config      

  config=kapp->config();
  printer = new QPrinter;
  untitledCount=0;
  pDocList = new QList<KsubeditorDoc>();
  pDocList->setAutoDelete(true);
  keyboard = new KAccel( this );
  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initView();
  initActions();
	
  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  filePrint->setEnabled(false);
  editCut->setEnabled(false);
  editCopy->setEnabled(false);
  editPaste->setEnabled(false);
}

KsubeditorApp::~KsubeditorApp()
{
  delete printer;
}

void KsubeditorApp::initActions()
{
  fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());

  windowNewWindow = new KAction(i18n("New &Window"), 0, this, SLOT(slotWindowNewWindow()), actionCollection(),"window_new_window");
  windowTile = new KAction(i18n("&Tile"), 0, this, SLOT(slotWindowTile()), actionCollection(),"window_tile");
  windowCascade = new KAction(i18n("&Cascade"), 0, this, SLOT(slotWindowCascade()), actionCollection(),"window_cascade");

  fileNew->setStatusText(i18n("Creates a new document"));
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileSave->setStatusText(i18n("Saves the actual document"));
  fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  fileClose->setStatusText(i18n("Closes the actual document"));
  filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));

  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));

  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));


  windowMenu = new KActionMenu(i18n("&Window"), actionCollection(), "window_menu");
  connect(windowMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(windowMenuAboutToShow()));

  createGUI();

  //other menus
  //subtitles menu
  sub_menu = new QPopupMenu(); 
  menuBar()->insertItem(i18n("&Subtitles"), sub_menu,0,2);
  connect( sub_menu, SIGNAL( aboutToShow() ),
             this, SLOT( slotSubtitlesMenuAboutToShow() ) );

  //options menu
  options_menu = new QPopupMenu();
  menuBar()->insertItem(i18n("&Options"), options_menu,0,3);
  options_menu->insertItem(SmallIcon("keyboard"),i18n("&Keyboard config"),this, SLOT(slotKeyConfig()));
  options_menu->insertItem(SmallIcon("configure"),i18n("&Ksubeditor config"),this, SLOT(slotConfig()));
  options_menu->insertItem(SmallIcon("configure"),i18n("&Spelling config"),this, SLOT(slotSpellConfig()));                                                          

  //keyboard support
 keyboard->insert( "Insert", i18n("Insert empty row above selected one."),
                       i18n("Insert empty row above selected one."),
                       Qt::Key_Insert, this, SLOT(slotInsert()) );
 keyboard->insert( "Insert_Under", i18n("Insert empty row under selected one."),
                       i18n("Insert empty row under selected one."),
                       Qt::CTRL+Key_Insert, this, SLOT(slotInsertUnder()) );
 keyboard->insert( "Delete", i18n("Delete selected row"),
                       i18n("Delete selected row."),
                       Qt::Key_Delete, this, SLOT(slotDelete()) );
 keyboard->insert( "Plus", i18n("Add one second to all subtitles below selected."),
                       i18n("Add one second to all subtitles below selected."),
                       Qt::Key_Equal, this, SLOT(slotPlusAll()) );
 keyboard->insert( "Minus", i18n("Back one second to all subtitles below selected."),
                       i18n("Subtract one second to all subtitles below selected."),
                       Qt::Key_Minus, this, SLOT(slotMinusAll()) );
 keyboard->insert( "Play", i18n("Start player."),
                       i18n("Start player to test subtitles."),
                       Qt::ALT+Key_P, this, SLOT(slotPlay()) );
 keyboard->insert( "Move_Line", i18n("Add one second to the current line."),
                       i18n("Add one second to the current line."),
                       Qt::Key_Plus, this, SLOT(slotPlusOne()) );
 keyboard->insert( "Move_Line2", i18n("Back one second to the current line."),
                       i18n("Back one second to the current line."),
                       Qt::Key_Underscore, this, SLOT(slotMinusOne()) );
 keyboard->insert( "Plus_Ms", i18n("Plus miliseconds (or frames) to current line."),
                       i18n("Plus miliseconds (or frames) to current line."),
                       Qt::ALT+Key_PageUp, this, SLOT(slotPlusMs()) );
 keyboard->insert( "Minus_Ms", i18n("Minus miliseconds (or frames) to current line."),
                       i18n("Minus miliseconds (or frames) to current line."),
                       Qt::ALT+Key_PageDown, this, SLOT(slotMinusMs()) );
 keyboard->insert( "Plus_MsAll", i18n("Plus miliseconds (or frames) to all subtitles below selected."),
                       i18n("Plus miliseconds (or frames) to all subtitles below selected."),
                       Qt::CTRL+Key_PageUp, this, SLOT(slotPlusMsAll()) );
 keyboard->insert( "Minus_MsAll", i18n("Minus miliseconds (or frames) to all subtitles below selected."),
                       i18n("Minus miliseconds (or frames) to all subtitles below selected."),
                       Qt::CTRL+Key_PageDown, this, SLOT(slotMinusMsAll()) );


 toolBar()->insertSeparator();
 toolBar()->insertButton(BarIcon("charset"), ID_CHARSET,SIGNAL(clicked()), this, SLOT(slotConvertCharset()),true, i18n("Convert Polish letters from windows into ISO-8559-2") );
 toolBar()->insertButton(BarIcon("xclock"), ID_MOVE_TIME,SIGNAL(clicked()), this, SLOT(slotMoveSubtitles()),true, i18n("Move subtitles time") );
 toolBar()->insertButton(BarIcon("player_play"), ID_PLAY,SIGNAL(clicked()), this, SLOT(slotPlay()),true, i18n("Test in mplayer") );                                                                                                                                                                

 
    
}


void KsubeditorApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."),1);
}


void KsubeditorApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // here the main view of the KMainWindow is created by a background box and
  // the QWorkspace instance for MDI view.
  QVBox* view_back = new QVBox( this );
  view_back->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
  pWorkspace = new QWorkspace( view_back );
  setCentralWidget(view_back);

}


void KsubeditorApp::createClient(KsubeditorDoc* doc)
{
  KsubeditorView* w = new KsubeditorView(doc, pWorkspace,0,WDestructiveClose);
  w->installEventFilter(this);
  doc->addView(w);
  w->setIcon(kapp->miniIcon());
  if ( pWorkspace->windowList().isEmpty() ) // show the very first window in maximized mode
    w->showMaximized();
  else
    w->show();
  connect(w, SIGNAL(got_focus()), this, SLOT(slotUpdateKeyAccel()));
}

void KsubeditorApp::openDocumentFile(const KURL& url,int format,bool nowy)
{
  slotStatusMsg(i18n("Opening file..."));
  KsubeditorDoc* doc;
	// check, if document already open. If yes, set the focus to the first view
  for(doc=pDocList->first(); doc > 0; doc=pDocList->next())
  {
    if(doc->URL()==url)
    {
      KsubeditorView* view=doc->firstView();
      view->setFocus();
      return;
    }
  }
  doc = new KsubeditorDoc();
  doc->format = format;
  pDocList->append(doc);
  doc->newDocument();
  // Creates an untitled window if file is 0
  if(url.isEmpty())
  {
    untitledCount+=1;
    QString fileName=QString(i18n("Untitled%1")).arg(untitledCount);
    KURL url;
    url.setFileName(fileName);
    doc->setURL(url);
    createClient(doc);
    doc->openDocument(url,format);
    if (nowy) doc->insert(0);
  }
  // Open the file
  else
  {
 ///  QString scie;
//   scie = url.url();
//   KIO::NetAccess::download( url, scie );
   doc->setURL(url);
   createClient(doc);
   if (!doc->openDocument(url))
   {
       doc->closeDocument();
       KMessageBox::error (0,i18n("Could not open document !"), i18n("Error !"));
       return;
;  }
 	 fileOpenRecent->addURL(url);
  } 

  
  toolBar()->addConnection(ID_CHARSET,SIGNAL(clicked()),doc,SLOT(slotConvertCharset()));
  toolBar()->addConnection(ID_MOVE_TIME,SIGNAL(clicked()),doc,SLOT(slotMoveTime()));
  toolBar()->addConnection(ID_PLAY,SIGNAL(clicked()),doc,SLOT(slotPlay()));

  connect(doc->napisy->Lista, SIGNAL( contextMenuRequested( QListViewItem *, const QPoint& , int ) ),
          doc, SLOT( slotRightClick(QListViewItem *, const QPoint &, int ) ) );

  connect(doc->napisy->Lista, SIGNAL( returnPressed( QListViewItem *) ),
          doc, SLOT( slotMultiLineEdit(QListViewItem *) ) );

  connect(doc->napisy->Lista, SIGNAL( doubleClicked( QListViewItem *, const QPoint& , int ) ),
           doc, SLOT( slotEdit(QListViewItem *, const QPoint &, int ) ) );                        

  connect(doc->napisy->Lista, SIGNAL( itemRenamed( QListViewItem *, int ) ),
          doc, SLOT( slotModified() ) );            
           
  // create the window
  slotStatusMsg(i18n("Ready."));
   if (!doc->sub_uses_time) {
    char txt[40];
      sprintf(txt,"%f",doc->napisy->fps);
      statusBar()->changeItem(i18n("Subtitles FPS: ")+txt,3);
   } else statusBar()->changeItem(i18n("Subtitles FPS: Not available"),3);
}


void KsubeditorApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", toolBar()->isVisible());
  config->writeEntry("Show Statusbar",statusBar()->isVisible());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  //user config items
  config->writeEntry("Path to Mplayer",KsubeditorConfig->MplayerPath);
  config->writeEntry("Default font encoding",KsubeditorConfig->defaultEncoding);
  config->writeEntry("Miliseconds to move in SRT",KsubeditorConfig->move_SRT);
  config->writeEntry("Frames to move in MicroDVD",KsubeditorConfig->move_MicroDVD);

  fileOpenRecent->saveEntries(config,"Recent Files");
}


void KsubeditorApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  //user config items
  KsubeditorConfig->MplayerPath = config->readPathEntry("Path to Mplayer");
  KsubeditorConfig->defaultEncoding = config->readNumEntry("Default font encoding");
  KsubeditorConfig->move_SRT = config->readNumEntry("Miliseconds to move in SRT");
  KsubeditorConfig->move_MicroDVD = config->readNumEntry("Frames to move in MicroDVD");

  
  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

void KsubeditorApp::saveProperties(KConfig *_cfg)
{

}


void KsubeditorApp::readProperties(KConfig* _cfg)
{
}

bool KsubeditorApp::queryClose()
{

  QStringList saveFiles;
  KsubeditorDoc* doc;
  if(pDocList->isEmpty())
    return true;

  for(doc=pDocList->first(); doc!=0;doc=pDocList->next())
  {
    if(doc->isModified())
      saveFiles.append(doc->URL().fileName());
  }
  if(saveFiles.isEmpty())
    return true;

  switch (KMessageBox::questionYesNoList(this,
          i18n("One or more documents have been modified.\nSave changes before exiting?"),saveFiles))
  {
  case KMessageBox::Yes:
    for(doc=pDocList->first(); doc!=0;doc=pDocList->next())
    {
      if(doc->URL().fileName().contains(i18n("Untitled")))
        slotFileSaveAs();
      else
      {
        if(!doc->saveDocument(doc->URL()))
        {
          KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
          return false;
         }
       }
     }
     return true;
  case KMessageBox::No:
  default:
  return true;
  }
}

bool KsubeditorApp::queryExit()
{
  saveOptions();
  return true;
}

bool KsubeditorApp::eventFilter(QObject* object, QEvent* event)
{
  if((event->type() == QEvent::Close)&&((KsubeditorApp*)object!=this))
  {
    QCloseEvent* e=(QCloseEvent*)event;

    KsubeditorView* pView=(KsubeditorView*)object;
    KsubeditorDoc* pDoc=pView->getDocument();
    if(pDoc->canCloseFrame(pView))
    {
      pDoc->removeView(pView);
      if(!pDoc->firstView())
        pDocList->remove(pDoc);
      e->accept();
    }
    else
      e->ignore();
  }
  return QWidget::eventFilter( object, event );    // standard event processing
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void KsubeditorApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));
  int format = choseFormat();

  if (format!=SUB_INVALID)
  {
    openDocumentFile(0,format,true);
  }  

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  KURL url=KFileDialog::getOpenURL(QString::null,
      i18n("*|All files"), this, i18n("Open File..."));
  if(!url.isEmpty())
  {
    openDocumentFile(url);
    fileOpenRecent->addURL( url );
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
  	
  openDocumentFile(url);
	
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));

  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
  if( m )
  {
    KsubeditorDoc* doc = m->getDocument();
    if(doc->URL().fileName().contains(i18n("Untitled")))
      slotFileSaveAs();
    else
      if(!doc->saveDocument(doc->URL()))
    KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*|All files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
    if( m )
    {
      KsubeditorDoc* doc =	m->getDocument();
      if(!doc->saveDocument(url))
      {
        KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
        return;
      }
      doc->changedViewList();
//      setWndTitle(m);
	    fileOpenRecent->addURL(url);
    }	
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
  if( m )
  {
    KsubeditorDoc* doc=m->getDocument();
    doc->closeDocument();
  }
	
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
    m->print( printer );

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
        break;
    }
  }	
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotEditUndo()
{
  slotStatusMsg(i18n("Reverting last action..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
//  m->undo();

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
//    m->cut();	

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
//    m->copy();
		
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
//    m->paste();
		
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggle the toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotWindowNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));

  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
  {
    KsubeditorDoc* doc = m->getDocument();
    createClient(doc);
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotWindowTile(){
  pWorkspace->tile();
}

void KsubeditorApp::slotWindowCascade(){
  pWorkspace->cascade();
}

void KsubeditorApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text,1);
}


void KsubeditorApp::windowMenuAboutToShow()
{
  windowMenu->popupMenu()->clear();
  windowMenu->insert(windowNewWindow);
  windowMenu->insert(windowCascade);
  windowMenu->insert(windowTile);

  if ( pWorkspace->windowList().isEmpty() ){
    windowNewWindow->setEnabled(false);
    windowCascade->setEnabled(false);
    windowTile->setEnabled(false);
  }
  else{
    windowNewWindow->setEnabled(true);
    windowCascade->setEnabled(true);
    windowTile->setEnabled(true);
  }
  windowMenu->popupMenu()->insertSeparator();

  QWidgetList windows = pWorkspace->windowList();
  for ( int i = 0; i < int(windows.count()); ++i )
  {
    int id = windowMenu->popupMenu()->insertItem(QString("&%1 ").arg(i+1)+windows.at(i)->caption(), this, SLOT( windowMenuActivated( int ) ) );
    windowMenu->popupMenu()->setItemParameter( id, i );
    windowMenu->popupMenu()->setItemChecked( id, pWorkspace->activeWindow() == windows.at(i) );
  }
}

void KsubeditorApp::windowMenuActivated( int id )
{
  QWidget* w = pWorkspace->windowList().at( id );
  if ( w )
    w->setFocus();
}

/** No descriptions */
void KsubeditorApp::slotSubtitlesMenuAboutToShow(){
  sub_menu->clear();
  int mv_time = sub_menu->insertItem(SmallIcon("xclock"),i18n("&Move time"),this, SLOT(slotMoveSubtitles()));
  int ch_fps = sub_menu->insertItem(SmallIcon("video"),i18n("&Change FPS"));
  sub_menu->insertItem(SmallIcon("gear"),i18n("&Change subtitles format"),this, SLOT(slotChangeFormat()));
  sub_menu->insertItem(SmallIcon("charset"),i18n("&Convert polish chars"),this, SLOT(slotConvertCharset()));
  sub_menu->insertItem(SmallIcon("player_play"),i18n("&Test in mplayer"),this, SLOT(slotPlay()));
  sub_menu->insertItem(SmallIcon("revert"),i18n("&Insert file"),this, SLOT(slotInsertFile()));
  sub_menu->insertItem(SmallIcon("view_top_bottom"),i18n("&Split to files"),this, SLOT(slotSplit()));
  sub_menu->insertItem(SmallIcon("spellcheck"),i18n("&Spell checking"),this, SLOT(slotSpellCheck()));
  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
  if( m )
  {    
    sub_menu->connectItem(mv_time,m->getDocument(),SLOT(slotMoveTime()));
    sub_menu->connectItem(ch_fps,m->getDocument(),SLOT(slotChangeFPS()));

  }
}/** No descriptions */

void KsubeditorApp::slotChangeFormat()
{


  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

  if( m )
  {
      KsubeditorDoc* doc = m->getDocument();
      int new_form = choseFormat();
      
      if ((new_form!=SUB_INVALID)&&(doc->format!=new_form))
      {
          openDocumentFile(0,new_form);
          KsubeditorView* m2 = (KsubeditorView*)pWorkspace->activeWindow();
          KsubeditorDoc* doc2 = m2->getDocument();
          doc2->setModified(true);

          char txt[12];
          sprintf(txt,"%f",doc2->napisy->fps);
          statusBar()->changeItem(i18n("Subtitles FPS: ")+txt,3);
          doc->napisy->convert(new_form,m2->ListView1,doc2->napisy->fps);
          doc2->napisy->check();
      }
  }
}

/** No descriptions */
void KsubeditorApp::slotKeyConfig()
{

KKeyDialog::configure(keyboard);

}

/** No descriptions */
void KsubeditorApp::slotConfig()
{

  configDialog->mplayer->setURL(KsubeditorConfig->MplayerPath);
  configDialog->encoding->setCurrentItem(KsubeditorConfig->defaultEncoding);
  configDialog->move_SRT->setValue(configuration::move_SRT);
  configDialog->move_MicroDVD->setValue(configuration::move_MicroDVD);

  if (configDialog->exec())
  {
    KsubeditorConfig->MplayerPath = configDialog->mplayer->url();
    KsubeditorConfig->defaultEncoding = configDialog->encoding->currentItem();
    KsubeditorConfig->move_SRT =  configDialog->move_SRT->value();
    KsubeditorConfig->move_MicroDVD =  configDialog->move_MicroDVD->value();
  }

}
/** No descriptions */
void KsubeditorApp::slotSplit()
{

 KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
 {
   if(m->getDocument()->napisy->Lista->selectedItem())
   {
      QListViewItem *item2,*item3;
      openDocumentFile(0,m->getDocument()->format);
      m->getDocument()->setModified(true);
      KsubeditorView* m2 = (KsubeditorView*)pWorkspace->activeWindow();
      KsubeditorDoc *doc = m2->getDocument();
      doc->setModified(true);
      
      item2 = 0;

   /*   for (int a=0;a<m->getDocument()->napisy->Lista->columns();a++)
      {
        doc->napisy->Lista->addColumn(m->getDocument()->napisy->Lista->columnText(a));
      }
*/
      doc->napisy->Lista->setSorting(-1,false);

      for(QListViewItem *item=m->getDocument()->napisy->Lista->selectedItem();
            item!=0;)
      {
        if (item2) item2 = new KListViewItem( doc->napisy->Lista, item2 );
           else item2 = new KListViewItem( doc->napisy->Lista, 0 );

        for (int b = 0;b<doc->napisy->Lista->columns();b++)
        {
          item2->setText(b,item->text(b));
        }

        item3 = item;
        item = item->itemBelow();
        m->getDocument()->napisy->Lista->takeItem(item3);
      }
      doc->napisy->setRenaming();
    } else
          KMessageBox::error (this,
          i18n("You have to select a splitting line!"),
           i18n("Where to split?"));
   
  }

}
/** No descriptions */
void KsubeditorApp::slotInsertFile()
{

   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
    int format = -1;
    int format2= -1;

    KURL url=KFileDialog::getOpenURL(QString::null,
      i18n("*|All files"), this, i18n("Open File..."));

    if(!url.isEmpty())
    {
      format2 = m->getDocument()->format;
      format = m->getDocument()->typeDetect(url.path().latin1());
      m->getDocument()->setModified(true);
      if (format==format2)
      {
        m->getDocument()->napisy->inserting = true;
        m->getDocument()->napisy->load(url.path().latin1());
        m->getDocument()->napisy->inserting = false;
      } else
         KMessageBox::error (this,i18n("Incompatible subtitle formats !"),
           i18n("Could not join files"));
    }
  }
}

/** No descriptions */
int KsubeditorApp::choseFormat()
{
    int form = SUB_INVALID;
    
    conv_sub = new convert(this,"Change",true);

    if (conv_sub->exec())
    {


      if (conv_sub->microDVD->isChecked())
          form =  SUB_MICRODVD;
      if (conv_sub->SRT->isChecked())
          form =  SUB_SUBRIP;
      if (conv_sub->vplayer->isChecked())
          form =  SUB_VPLAYER;
      if (conv_sub->ssa->isChecked())
          form =  SUB_SSA;
     }

     delete conv_sub;

     return form;  
}
/** No descriptions */
void KsubeditorApp::slotSpellConfig()
{

  QTabDialog qtabdialog (0, 0, true);
  qtabdialog.setCancelButton();
  KSpellConfig ksconfig (&qtabdialog);
  qtabdialog.addTab (&ksconfig, "&Spellchecker");
  if (qtabdialog.exec())
    ksconfig.writeGlobalSettings();

}

/** No descriptions */
void KsubeditorApp::spell_started(KSpell *)
{

   kspell->setProgressResolution(2);
   kspell->check(temp_edit->text());

}

/** No descriptions */
void KsubeditorApp::spell_done(const QString& newtext)
{
  temp_edit->spellcheck_stop();

  if (kspell->dlgResult() == 0)
  {
     temp_edit->setText( newtext);
     KMessageBox::information(0, i18n("SpellCheck Aborted"));

  }
  else
  {
     KMessageBox::information(0, i18n("SpellCheck Complete"));

  }
  kspell->cleanUp();
}/** No descriptions */

void KsubeditorApp::spell_finished()
{

  KSpell::spellStatus status = kspell->status();
  delete kspell;
  kspell = 0;

  if (status == KSpell::Error)
  {
     KMessageBox::sorry(0, i18n("ISpell could not be started.\n"
     "Please make sure you have ISpell properly configured and in your PATH."));
  }
  else if (status == KSpell::Crashed)
  {
     temp_edit->spellcheck_stop();

     KMessageBox::sorry(0, i18n("ISpell seems to have crashed."));
  }

   int b = 0;

   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
     for (QListViewItem  * item = m->getDocument()->napisy->Lista->firstChild();
                    item!=0;item = item->itemBelow())
     {
       for (int a=m->getDocument()->napisy->col;a<m->getDocument()->napisy->col+3;a++)
       {
        item->setText(a,temp_edit->textLine(b++));
       }

      }
  }
  delete temp_edit;
}

/** No descriptions */
void KsubeditorApp::slotSpellCheck(){



   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
     temp_edit = new KEdit(0,"Temporary edit for spell check");
     temp_edit->setFixedSize(400,200);
     int b = 0;

     for (QListViewItem  * item = m->getDocument()->napisy->Lista->firstChild();
                    item!=0;item = item->itemBelow())
     {
       for (int a=m->getDocument()->napisy->col;a<m->getDocument()->napisy->col+3;a++)
       {
        temp_edit->insertLine(item->text(a),b++);
       }

     }

      temp_edit->show();

      kspell = new KSpell(0, i18n("Spellcheck"), this,
            SLOT( spell_started(KSpell *)),0,true,true);//, &mOptionState.spell.config);

      connect (kspell, SIGNAL ( death()),
              this, SLOT ( spell_finished( )));

      connect (kspell, SIGNAL (done(const QString&)),
                     this, SLOT (spell_done(const QString&)));

      connect (kspell, SIGNAL (progress (unsigned int)),
              this, SLOT (spell_progress (unsigned int)));
      connect (kspell, SIGNAL (misspelling (const QString &, const QStringList &, unsigned int)),
               temp_edit, SLOT (misspelling (const QString &, const QStringList &, unsigned int)));
      connect (kspell, SIGNAL (corrected (const QString &, const QString &, unsigned int)),
               temp_edit, SLOT (corrected (const QString &, const QString &, unsigned int)));
  }

}




/** No descriptions */
void KsubeditorApp::slotInsert(bool under,QListViewItem *to_insert)
{

  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {

      m->getDocument()->insert(false,to_insert);

   }
}

/** No descriptions */
void KsubeditorApp::slotDelete(QListViewItem * to_delete)
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->delete_(to_delete);

   }
}


/** No descriptions */
void KsubeditorApp::slotPlusAll()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->napisy->moveTime(1,false);
      m->getDocument()->setModified(true);
   }   

}
/** No descriptions */
void KsubeditorApp::slotMinusAll()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

       m->getDocument()->napisy->moveTime(-1,false);
       m->getDocument()->setModified(true);
   }   

}
/** No descriptions */
void KsubeditorApp::slotPlusOne()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->napisy->moveTime(1,true);
      m->getDocument()->setModified(true);
   }
}
/** No descriptions */
void KsubeditorApp::slotMinusOne()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->napisy->moveTime(-1,true);
      m->getDocument()->setModified(true);
   }    

}
/** No descriptions */
void KsubeditorApp::slotPlay()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->play();

   }   
 
}


/** No descriptions */
void KsubeditorApp::slotPlusMs()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->napisy->moveMs(m->getDocument()->napisy->move,true);
      m->getDocument()->setModified(true);
   }   

}
/** No descriptions */
void KsubeditorApp::slotMinusMs()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->napisy->moveMs(-m->getDocument()->napisy->move,true);
      m->getDocument()->setModified(true);
   }   

}
/** No descriptions */
void KsubeditorApp::slotPlusMsAll()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->napisy->moveMs(m->getDocument()->napisy->move,false);
      m->getDocument()->setModified(true);
   }
}
/** No descriptions */
void KsubeditorApp::slotMinusMsAll()
{
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->napisy->moveMs(-m->getDocument()->napisy->move,false);
      m->getDocument()->setModified(true);
   }
}


/*!
    \fn KsubeditorApp::slotInsertUnder()
 */
void KsubeditorApp::slotInsertUnder()
{
	KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();

   if( m )
   {

      m->getDocument()->insert(true);

   }

}
