/***************************************************************************
                          ksubeditordoc.cpp  -  description
                             -------------------
    begin                : pi sty 10 21:05:38 CET 2003
    copyright            : (C) 2003 by Anatol Ogrek
    email                : mroova@linux.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define READ_ERROR    -2
#define SUB_INVALID   -1
#define SUB_MICRODVD  0
#define SUB_SUBRIP    1
#define SUB_SUBVIEWER 2
#define SUB_SAMI      3
#define SUB_VPLAYER   4
#define SUB_RT        5
#define SUB_SSA       6
#define SUB_DUNNOWHAT 7		// FIXME what format is it ?
#define SUB_MPSUB     8
#define SUB_AQTITLE   9
#define SUB_MAX_TEXT  3
#define LINE_LEN  1000
#define ID_PLAY 14000
#define ID_MOVE_TIME 14010
#define ID_CHARSET 14020 

 
// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qmultilineedit.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <ktempfile.h>
#include <keditcl.h>

// application specific includes
#include "ksubeditor.h"
#include "ksubeditorview.h"
#include "ksubeditordoc.h"

//other includes
#include "subtitles.h"
#include "microdvd.h"
#include "vplayer.h"
#include "subripper.h"
#include "ssa.h"
#include <stdlib.h>

#include "configuration.h"
#include "edit.h"

const int charcount = 18;
const QChar charset_org[charcount]= {353,263,281,322,324,243,156,380,159,317,262,280,321,323,211,140,379,143};
const QChar charset_new[charcount]= {261,263,281,322,324,243,347,380,378,260,262,280,321,323,211,346,379,377};


KsubeditorDoc::KsubeditorDoc()
{
  pViewList = new QList<KsubeditorView>;
  pViewList->setAutoDelete(false);

}

KsubeditorDoc::~KsubeditorDoc()
{
  delete pViewList;
}

void KsubeditorDoc::addView(KsubeditorView *view)
{
  pViewList->append(view);
	changedViewList();
}

void KsubeditorDoc::removeView(KsubeditorView *view)
{
	  pViewList->remove(view);
	  if(!pViewList->isEmpty())
			changedViewList();
		else
			deleteContents();
}

void KsubeditorDoc::changedViewList(){	
	
	KsubeditorView *w;
	if((int)pViewList->count() == 1){
  	w=pViewList->first();
  	w->setCaption(URL().fileName());
	}
	else{	
		int i;
    for( i=1,w=pViewList->first(); w!=0; i++, w=pViewList->next())
  		w->setCaption(QString(URL().fileName()+":%1").arg(i));	
	}
}

bool KsubeditorDoc::isLastView() {
  return ((int) pViewList->count() == 1);
}


void KsubeditorDoc::updateAllViews(KsubeditorView *sender)
{
  KsubeditorView *w;
  for(w=pViewList->first(); w!=0; w=pViewList->next())
  {
     w->update(sender);
  }

}

void KsubeditorDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& KsubeditorDoc::URL() const
{
  return doc_url;
}

void KsubeditorDoc::closeDocument()
{
  KsubeditorView *w;
  if(!isLastView())
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
   	 	if(!w->close())
 				break;
    }
	}
  if(isLastView())
  {
  	w=pViewList->first();
  	w->close();
  }
}

bool KsubeditorDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  return true;
}

bool KsubeditorDoc::openDocument(const KURL &url, int format /*=0*/)
{
   int code;
   if(typeDetect(url.path().latin1())==READ_ERROR)
   {

        code = format;
   }   
    else code=typeDetect(url.path().latin1());

   if ( code>=0)
   { //if not error
	 this->format = format;
     switch (code){
       case SUB_MICRODVD: napisy = new microDVD(pViewList->last()->ListView1);                   
                          break;
       case SUB_VPLAYER:  napisy = new vplayer(pViewList->last()->ListView1);
                          break;                          
       case SUB_SUBRIP:   napisy = new subRipper(pViewList->last()->ListView1);
                          break;
       case SUB_SSA:   napisy = new ssa(pViewList->last()->ListView1);
                          break;
                          
       default: break;
     }
     napisy->setColumns();

   return napisy->load(url.path().latin1());
   }
   else
   {
     switch (code)
     {
         case SUB_INVALID: KMessageBox::error (0,i18n("Invalid subtitles format !"), i18n("Error !"));
                            return false;
                            break;
         case READ_ERROR:  KMessageBox::error (0,i18n("Bad file or no read permissions !"), i18n("Error !"));
                            return false;
                            break;
     }
   }  
return true;
}

bool KsubeditorDoc::saveDocument(const KURL &url /*=0*/,bool temp)
{

  if (!napisy->save(url.path().latin1()))
  {
     KMessageBox::error (0,i18n("Problem Writting File !"), i18n("Error !"));
     return false;
  }   
  else
  {
    if (temp) {
		setModified(false);
		setURL(url);
	}
  }
    return true;
}

void KsubeditorDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}

bool KsubeditorDoc::canCloseFrame(KsubeditorView* pFrame)
{
	if(!isLastView())
		return true;
		
	bool ret=false;
  if(isModified())
  {
		KURL saveURL;
  	switch(KMessageBox::warningYesNoCancel(pFrame, i18n("The current file has been modified.\n"
                          "Do you want to save it?"),URL().fileName()))
    {
			case KMessageBox::Yes:
				if(URL().fileName().contains(i18n("Untitled")))
				{
					saveURL=KFileDialog::getSaveURL(QDir::currentDirPath(),
                      i18n("*|All files"), pFrame, i18n("Save as..."));
          if(saveURL.isEmpty())
          	return false;
				}
				else
						saveURL=URL();
					
				if(!saveDocument(saveURL))
				{
 					switch(KMessageBox::warningYesNo(pFrame,i18n("Could not save the current document !\n"
																												"Close anyway ?"), i18n("I/O Error !")))
 					{
 						case KMessageBox::Yes:
 							ret=true;
 						case KMessageBox::No:
 							ret=false;
 					}	        			
				}
				else
					ret=true;
				break;
			case KMessageBox::No:
				ret=true;
				break;
			case KMessageBox::Cancel:
			default:
				ret=false; 				
				break;
		}
	}
	else
		ret=true;
		
	return ret;
}

int KsubeditorDoc::typeDetect(const char * fname){

    char line[LINE_LEN+1];
    int i,j=0;
    char p;
    FILE * fd;
    sub_uses_time = 1;
    fd=fopen (fname, "r"); if (!fd) return -2;        //read error
    while (j < 10) {
	j++;
	if (!fgets (line, LINE_LEN, fd))
	 { fclose(fd);  format = SUB_INVALID;  return SUB_INVALID;}

	if (sscanf (line, "{%d}{%d}", &i, &i)==2)
		{sub_uses_time=0; fclose(fd); format = SUB_MICRODVD;return SUB_MICRODVD;}
	if (sscanf (line, "{%d}{}", &i)==1)
		{sub_uses_time=0; fclose(fd); format = SUB_MICRODVD;return SUB_MICRODVD;}
	if (sscanf (line, "%d:%d:%d.%d,%d:%d:%d.%d",     &i, &i, &i, &i, &i, &i, &i, &i)==8)
		{sub_uses_time=1; fclose(fd); format = SUB_SUBVIEWER;return SUB_SUBVIEWER;}
	if (sscanf (line, "%d:%d:%d,%d --> %d:%d:%d,%d", &i, &i, &i, &i, &i, &i, &i, &i)==8)
		{sub_uses_time=1; fclose(fd); format = SUB_SUBRIP;return SUB_SUBRIP;}
	if (strstr (line, "<SAMI>"))
		{sub_uses_time=1; fclose(fd);  format = SUB_SAMI;return SUB_SAMI;}
	if (sscanf (line, "%d:%d:%d:",     &i, &i, &i )==3)
		{sub_uses_time=1; fclose(fd); format = SUB_VPLAYER;return SUB_VPLAYER;}
	if (sscanf (line, "%d:%d:%d ",     &i, &i, &i )==3)
		{sub_uses_time=1; fclose(fd); format = SUB_VPLAYER;return SUB_VPLAYER;}
	//TODO: just checking if first line of sub starts with "<" is WAY
	// too weak test for RT
	// Please someone who knows the format of RT... FIX IT!!!
	// It may conflict with other sub formats in the future (actually it doesn't)
	if ( *line == '<' )
		{sub_uses_time=1; fclose(fd); format = SUB_RT;return SUB_RT;}

	if (!memcmp(line, "Dialogue: Marked", 16))
		{sub_uses_time=1; fclose(fd);  format = SUB_SSA;return SUB_SSA;}
	if (sscanf (line, "%d,%d,\"%c", &i, &i, (char *) &i) == 3)
		{sub_uses_time=0; fclose(fd); format = SUB_DUNNOWHAT;return SUB_DUNNOWHAT;}
	if (sscanf (line, "FORMAT=%d", &i) == 1)
		{sub_uses_time=0; fclose(fd);  format = SUB_MPSUB;return SUB_MPSUB;}
	if (sscanf (line, "FORMAT=TIM%c", &p)==1 && p=='E')
		{sub_uses_time=1; fclose(fd);  format = SUB_MPSUB;return SUB_MPSUB;}
	if (strstr (line, "-->>"))
		{sub_uses_time=0; fclose(fd); format = SUB_MPSUB;return SUB_MPSUB;}
    }
    fclose(fd);
    format = SUB_INVALID;
    return SUB_INVALID;  // too many bad lines


}/** No descriptions */
void KsubeditorDoc::slotChangeFPS(){
   napisy->ChangeFPS();
   setModified(true);
}
/** No descriptions */

void KsubeditorDoc::slotMoveTime(){
  napisy->moveTime(0,false);
  setModified(true);  
}
/** No descriptions */
void KsubeditorDoc::slotConvertCharset()
{

   for (QListViewItem  * item = napisy->Lista->firstChild();
                  item!=0;item = item->itemBelow())
   {
     for (int a=napisy->col;a<napisy->col+3;a++)
     {
       QString linia = item->text(a);
       for (unsigned int b = 0;b<linia.length();b++)
       {
         for (int c = 0;c<charcount;c++)
         {
           QChar l = linia[b];
           if (l == charset_org[c]) linia[b] = charset_new[c];
         }
       }
       item->setText(a,linia);
     }
   }   
   setModified(true);
   
}
/** No descriptions */
void KsubeditorDoc::slotRightClick(QListViewItem* item, const QPoint & point, int)
{
  int i;

   right_button_menu = new QPopupMenu(napisy->Lista);

   int DELETE = right_button_menu->insertItem(i18n("Delete row"));
   int INSERT = right_button_menu->insertItem(i18n("Insert row"));

   if (item)   i = right_button_menu->exec(point);
      else
          return;

   if (i == -1) return;

   if (i == DELETE) {
                    delete_(item);
                    setModified(true);

                    }
   if (i == INSERT) {
                    insert(item);
                    setModified(true);     
                    }


}
/** No descriptions */
void KsubeditorDoc::slotMultiLineEdit(QListViewItem * item)
{

 edit * editLine = new edit(0,"Edit");
 editLine->lineEdit->insertLine(item->text(napisy->col),0);
 editLine->lineEdit->insertLine(item->text(napisy->col+1),1);
 editLine->lineEdit->insertLine(item->text(napisy->col+2),2);
 if (editLine->exec())
 {
   item->setText(napisy->col,editLine->lineEdit->textLine(0));
   item->setText(napisy->col+1,editLine->lineEdit->textLine(1));
   item->setText(napisy->col+2,editLine->lineEdit->textLine(2));
 };

 delete editLine;
 setModified(true);

}


/** No descriptions */
void KsubeditorDoc::slotEdit(QListViewItem * item, const QPoint &, int c)
{

  item->startRename(c);
  setModified(true);


}
/** No descriptions */
void KsubeditorDoc::slotModified()
{

  napisy->check();
  setModified(true);

}

/** No descriptions */
void KsubeditorDoc::insert(bool under,QListViewItem *to_insert)
{

      KListViewItem * item;
      int a=1;

        if (!to_insert)
        {
          if (!napisy->Lista->firstChild())
             item = new KListViewItem( napisy->Lista, 0 );
          else
          {
			if (!under)
			{
				if (napisy->Lista->currentItem()->itemAbove())
	              a = napisy->Lista->currentItem()->itemAbove()->text(0).toInt()+1;
            	else a = 1;
            	item = new KListViewItem( napisy->Lista, napisy->Lista->currentItem()->itemAbove() );
			}
			else
			{
			 a = napisy->Lista->currentItem()->text(0).toInt()+1;
			 item = new KListViewItem(napisy->Lista,napisy->Lista->currentItem());
			}
          }
        }
        else
        {
		  if (!under)
		  {
		  	if (to_insert->itemAbove())
	            a = to_insert->itemAbove()->text(0).toInt()+1;
          	else a = 1;
          	item = new KListViewItem( napisy->Lista, to_insert->itemAbove() );
		  }
		  else
		  {
			//a =
		  }
        }



  /*      item->setRenameEnabled(0,true);
        item->setRenameEnabled(1,true);
        item->setRenameEnabled(2,true);
        item->setRenameEnabled(3,true);
        item->setRenameEnabled(4,true);
*/
		napisy->setRenaming();
        napisy->Lista->insertItem(item);
        setModified(true);

        int b = 0;
        if (format == SUB_SUBRIP)
        {
          b = 1;
          for (QListViewItem  * item2 = item;
              item2!=0;item2 = item2->itemBelow())
          {
              item2->setText(0,QString("%1").arg(a++,0,10));
          }
        }

        for (int c = b;c<napisy->col;c++)
        {
           if (item->itemAbove())
             item->setText(c,item->itemAbove()->text(c));
           else
           if (item->itemBelow())
             item->setText(c,item->itemBelow()->text(c));
           else
             item->setText(c,"0");
        }

}
/** No descriptions */
void KsubeditorDoc::delete_(QListViewItem * to_delete)
{

 int a = 1;
   QListViewItem *item3,*item;

   if (!to_delete)
   {
     if (napisy->Lista->currentItem())
     {
       if (napisy->Lista->currentItem()->itemAbove())
       {
         a = napisy->Lista->currentItem()->itemAbove()->text(0).toInt();
         item3 = napisy->Lista->currentItem()->itemAbove();
       }
       else a = 1;
      }
       item = napisy->Lista->currentItem();
   }
   else item = to_delete;

   delete item;

   if (a==1) item3 = napisy->Lista->firstChild();
   setModified(true);

   if (format == SUB_SUBRIP)
      for (QListViewItem  * item2 = item3;
            item2!=0;item2 = item2->itemBelow())
      {
           item2->setText(0,QString("%1").arg(a++,0,10));
      }

}
/** No descriptions */
void KsubeditorDoc::play()
{

 QString command_line;

  if(movie_file.isEmpty())
  {
    movie_file=KFileDialog::getOpenURL(QString::null,
               i18n("*|All files"), 0,
               i18n("Chose path to the MOVIE file"));
  }

  if(!movie_file.isEmpty())
  {
    KTempFile * plik = new KTempFile();
    KURL url;
    url.setPath(plik->name());
    delete plik;
    saveDocument(url);
    QListViewItem * starting_row = napisy->Lista->selectedItem();


//      command_line = KsubeditorConfig->MplayerPath+" \"";
      command_line = "/usr/local/bin/mplayer \"";
      command_line+=movie_file.path()+"\" -sub \""+url.path()+"\"";
      command_line+=" -ss "+napisy->currentPosition(starting_row);
      system(command_line.latin1());
    }

}
