/***************************************************************************
                          microdvd.cpp  -  description
                             -------------------
    begin                : pi? sty 10 2003
    copyright            : (C) 2003 by Anatol Ogrek
    email                : mroova@linux.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define READ_ERROR    -2
#define SUB_INVALID   -1
#define SUB_MICRODVD  0
#define SUB_SUBRIP    1
#define SUB_SUBVIEWER 2
#define SUB_SAMI      3
#define SUB_VPLAYER   4
#define SUB_RT        5
#define SUB_SSA       6
#define SUB_DUNNOWHAT 7		// FIXME what format is it ?
#define SUB_MPSUB     8
#define SUB_AQTITLE   9
#define SUB_MAX_TEXT  3
#define LINE_LEN  1000
#define ID_PLAY 14000
#define ID_MOVE_TIME 14010
#define ID_CHARSET 14020 
 
//QT includes
#include "qcombobox.h"
#include "qfile.h"
#include "qradiobutton.h"
#include "qspinbox.h"

//KDE includes
#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <stdlib.h>   
#include "microdvd.h"
#include "configuration.h"

microDVD::microDVD(){
  inserting = false;
}
microDVD::~microDVD(){
}

/** Load subtitles in microDVD format */
int microDVD::load(QString filename){

    QString line1;
    QString line2;
    QString line3;
    int a,b,c,d;

    QListViewItem * item;

  if (!inserting)
  {
    item = 0;
  } else item = Lista->lastItem();

   QFile *fd = new QFile(filename);

   if (!fd->open(IO_ReadOnly)) return false;

   QTextStream t(fd);

   switch (configuration::defaultEncoding)
   {
       case 0:  break;
       case 1:  t.setEncoding(QTextStream::Unicode);
                break;
       case 2:  t.setEncoding(QTextStream::UnicodeReverse);
                break;
       case 3:  t.setEncoding(QTextStream::Latin1);
                break;
       default:
                t.setCodec(configuration::codecList->at(configuration::defaultEncoding-4) );
                break;
   }

   while (line1 = t.readLine())
   {
      a = line1.find("{",0);
      if (a == -1) continue;
      b = line1.find("}",a);
      if (b == -1) continue;
      line2 = line1.mid(a+1,b-a-1);
      c = line1.find("{",b-1);
      if (c == -1) continue;
      d = line1.find("}",c);
      if (d == -1) continue;
      line3 = line1.mid(c+1,d-c-1);

      if (item) item = new KListViewItem( Lista, item );
         else item = new KListViewItem( Lista, 0 );

      item->setRenameEnabled(0,true);
      item->setRenameEnabled(1,true);
      item->setRenameEnabled(2,true);
      item->setRenameEnabled(3,true);
      item->setRenameEnabled(4,true);

      item->setText( 0, tr2i18n( line2 ) );
      item->setText( 1, tr2i18n( line3 ) );

    //spread into lines

      line1=line1.right(line1.length()-d-1);
      a = 0;
      b = 0;
      c = 2;
      while (b!=line1.length())
      {

        b = line1.find('|',a);
        if (b==-1) b = line1.length();
        line2 = line1.mid(a,b-a);
        item->setText(c,line2);
        c++;
        a = b+1;
      }
   }
   return true;

}
/** No descriptions */
 microDVD::microDVD(KListView * Lista)
{

  inserting = false;
  move = configuration::move_MicroDVD;
  this->Lista = Lista;
  set_new_fps = new new_fps(this,"Set FPS",false);

  if (set_new_fps->exec())
     fps = atof(set_new_fps->ComboBox1->currentText());
  else fps = 25.0;

  delete set_new_fps;

  col = 2;
  
}
/** No descriptions */
bool microDVD::ChangeFPS(){

   char temp_fps[40];
   int set;

   convert_fps = new change_fps(this,"Change",true);

   if (convert_fps->exec())
   {

    float stosunek = atof(convert_fps->CurrentCombo->currentText())/atof(convert_fps->DestinationCombo->currentText());

    fps = atof(convert_fps->DestinationCombo->currentText());

    for (QListViewItem  * item = Lista->firstChild();
         item!=0;item = item->itemBelow())
    {

      set = atoi(item->text(0))/stosunek; //start frame

      sprintf(temp_fps,"%i",set);

      item -> setText(0,temp_fps);
      set = atoi(item->text(1))/stosunek; //stop frame
      sprintf(temp_fps,"%i",set);
      item -> setText(1,temp_fps);

    }
   }
}
/** No descriptions */
int microDVD::moveTime(int secs,bool one){

  long start,end;
  int  difference;
  QString line1;
  char ch_start[10];
  char ch_end[10];
  QListViewItem  * starting_row;

  movetime = new move_sub(this,"Move",true);

   if (!secs){
       if (movetime->exec())
       {

         if (movetime->RadioButton3->isChecked())
         {
           starting_row = Lista->firstChild();
         }
         if (movetime->RadioButton4->isChecked())
         {
           starting_row = Lista->selectedItem();
         }
         difference = movetime->second->value()+
                      movetime->minute->value()*60+
                      movetime->hour->value()*60;
         if (movetime->ComboBox1->currentText() == "Minus") difference = - difference;
        } else return true;
      } else {
               difference = secs ;
               if (!(starting_row = Lista->selectedItem())) starting_row = Lista->firstChild();
      }


     do {
       if (checkLine(starting_row)){
          start = atol(starting_row->text(0));
          end   = atol(starting_row->text(1));
          sprintf(ch_start,"%0.0f",start+difference*fps);
          sprintf(ch_end,"%0.0f",end+difference*fps);
          starting_row->setText(0,i18n(ch_start));
          starting_row->setText(1,i18n(ch_end));
       };

     } while ((starting_row=starting_row->itemBelow())&&(one == false));
    check();

  return true;
}

bool microDVD::checkLine(QListViewItem *line)
{
  bool ok = true;
  bool test = false;
  int a,len;

  if (line)
  {
   if (line->text(0))
   {

    line->text(0).toLong(&test,10);
    if (!test) return false;
    
   } else return false;
  } else return false;

  if (line->text(1))
  {
    line->text(1).toLong(&test,10);
    if (!test) return false;
  } else return false;

  return ok;
}

/** No descriptions */
bool microDVD::checkSorted(QListViewItem *item){
  int len,a1,a2,a3;
  int b1,b2,b3;
  char time[12];
  QListViewItem *item2;

  item2 = item->itemBelow();
  if (checkLine(item2))
  {
    if ((item->text(0))&&(item2->text(1)))
    {
       if (item2->text(0).toInt()<item->text(0).toInt()) return false;
       if (item2->text(1).toInt()<item->text(1).toInt()) return false;
     } else return false;
  } else return true;

  return true;

}

bool microDVD::save(QString filename)
{
  QFile * file = new QFile(filename);
  
  if (!file->open(IO_WriteOnly)) return false;
  QTextStream t(file);

  switch (configuration::defaultEncoding)
  {
    case 0:  break;
    case 1:  t.setEncoding(QTextStream::Unicode);
             break;
    case 2:  t.setEncoding(QTextStream::UnicodeReverse);
             break;
    case 3:  t.setEncoding(QTextStream::Latin1);
             break;
    default:
             t.setCodec(configuration::codecList->at(configuration::defaultEncoding-4) );
             break;
  }

  QString line;

  for (QListViewItem  * item = Lista->firstChild();
       item!=0;item = item->itemBelow())
  {
    line = "{"+item->text(0)+"}{"+item->text(1)+"}";
    if (item->text(2)) line+=item->text(2);
    if (item->text(3)) line+="|"+item->text(3);
    if (item->text(4)) line+="|"+item->text(4);
    line+="\r\n";
    t<<line;
  }
  file->close();

  return true;
}

/** No descriptions */
bool microDVD::convert(int dest,KListView *ListView, int fps2)
{

 KListViewItem  * dest_item;
 int a=1,b;
 dest_item = 0;

 switch (dest)
 {

   case SUB_VPLAYER:
       dest_item = 0;
       for (QListViewItem  * item = Lista->firstChild();
            item!=0;item = item->itemBelow())
       {

         if (dest_item) dest_item = new KListViewItem( ListView, dest_item );
             else dest_item = new KListViewItem( ListView, 0 );
         dest_item->setRenameEnabled(0,true);
         dest_item->setRenameEnabled(1,true);
         dest_item->setRenameEnabled(2,true);
         dest_item->setRenameEnabled(3,true);

         dest_item->setText(1,item->text(2));
         dest_item->setText(2,item->text(3));
         dest_item->setText(3,item->text(4));

         if (checkLine(item))
         {
            int seconds = atoi (item->text(0))/fps;
            char h[3],m[3],s[3];
            int a1,a2,a3;

            a1 = seconds/3600;
            a2 = (seconds - (a1*3600))/60;
            a3 = seconds - (a1*3600) - (a2*60);

            if (a1>9) sprintf(h,"%i",a1); else sprintf(h,"0%i",a1);
            if (a2>9) sprintf(m,"%i",a2); else sprintf(m,"0%i",a2);
            if (a3>9) sprintf(s,"%i",a3); else sprintf(s,"0%i",a3);
            dest_item->setText(0, tr2i18n(h)+":"+tr2i18n(m)+":"+tr2i18n(s));
        }
       }
       break;
  case SUB_SUBRIP:
  
        dest_item = 0;

        for (QListViewItem  * item = Lista->firstChild();
             item!=0;item = item->itemBelow())
        {

          if (dest_item) dest_item = new KListViewItem( ListView, dest_item );
              else dest_item = new KListViewItem( ListView, 0 );

          dest_item->setText(0,QString("%1").arg(a++,0,10));

          dest_item->setRenameEnabled(0,false);
          dest_item->setRenameEnabled(1,true);
          dest_item->setRenameEnabled(2,true);
          dest_item->setRenameEnabled(3,true);
          dest_item->setRenameEnabled(4,true);
          dest_item->setRenameEnabled(5,true);

          dest_item->setText(3,item->text(2));
          dest_item->setText(4,item->text(3));
          dest_item->setText(5,item->text(4));

          double seconds;
          char h[3],m[3],s[3],ms[4];
          int a1,a2,a3,a4;

          if (checkLine(item))
          {
            for (b = 0;b<2;b++)
            {
              seconds = item->text(b).toLong()/fps;

              QString str;
              a1 = seconds/3600;
              a2 = (seconds - (a1*3600))/60;
              a3 = seconds - (a1*3600) - (a2*60);

              str = QString("%1").arg(seconds,0,'f',3).right(3);
              if (a1>9) sprintf(h,"%i",a1); else sprintf(h,"0%i",a1);
              if (a2>9) sprintf(m,"%i",a2); else sprintf(m,"0%i",a2);
              if (a3>9) sprintf(s,"%i",a3); else sprintf(s,"0%i",a3);
              dest_item->setText(b+1, tr2i18n(h)+":"+tr2i18n(m)+":"+tr2i18n(s)+","+str);
            }
          }
        }
        break;
 case SUB_SSA:

        dest_item = 0;

        for (QListViewItem  * item = Lista->firstChild();
             item!=0;item = item->itemBelow())
        {

          if (dest_item) dest_item = new KListViewItem( ListView, dest_item );
              else dest_item = new KListViewItem( ListView, 0 );

          dest_item->setRenameEnabled(0,false);
          dest_item->setRenameEnabled(1,true);
          dest_item->setRenameEnabled(2,true);
          dest_item->setRenameEnabled(3,true);
          dest_item->setRenameEnabled(4,true);

          dest_item->setText(2,item->text(2));
          dest_item->setText(3,item->text(3));
          dest_item->setText(4,item->text(4));

          double seconds;
          char h[3],m[3],s[3],ms[4];
          int a1,a2,a3,a4;

          if (checkLine(item))
          {
            for (b = 0;b<2;b++)
            {
              seconds = item->text(b).toLong()/fps;

              QString str;
              a1 = seconds/3600;
              a2 = (seconds - (a1*3600))/60;
              a3 = seconds - (a1*3600) - (a2*60);

              str = QString("%1").arg(seconds,0,'f',3).right(3);
              if (a1>9) sprintf(h,"%i",a1); else sprintf(h,"0%i",a1);
              if (a2>9) sprintf(m,"%i",a2); else sprintf(m,"0%i",a2);
              if (a3>9) sprintf(s,"%i",a3); else sprintf(s,"0%i",a3);
              dest_item->setText(b, tr2i18n(h)+":"+tr2i18n(m)+":"+tr2i18n(s)+"."+str);
            }
          }
        }
        break;        
   default: return false;
            break;
   }
}
/** No descriptions */
QString microDVD::currentPosition(QListViewItem *item)
{
 QString seconds;
 if (checkLine(item))
   seconds.setNum(item->text(0).toInt()/fps-4);
                                           else seconds = "0";
 return seconds;                                           
}
/** No descriptions */
bool microDVD::moveMs(int frames, bool one)
{

  long start,end;

  QListViewItem  * starting_row;
  if (!(starting_row = Lista->selectedItem()))
      starting_row = Lista->firstChild();

  do
  {
    if (checkLine(starting_row))
    {
      starting_row->setText(0,i18n(QString("%1").arg(starting_row->text(0).toLong()+frames,0,10)));
      starting_row->setText(1,i18n(QString("%1").arg(starting_row->text(1).toLong()+frames,0,10)));
    };
  } while ((starting_row=starting_row->itemBelow())&&(one == false));

  check();

  return true;

}
/** No descriptions */
void microDVD::setColumns()
{

    Lista->addColumn("Start frame");
    Lista->addColumn("Stop frame");
    Lista->addColumn("First line");
    Lista->addColumn("Second line");
    Lista->addColumn("Third line");
    Lista->setSorting(-1,false);

}

/*!
    \fn microDVD::setRenaming()
 */
void microDVD::setRenaming()
{
 for (QListViewItem  * item = Lista->firstChild();
              item!=0;item = item->itemBelow())
	{
   		   item->setRenameEnabled(0,true);
           item->setRenameEnabled(1,true);
           item->setRenameEnabled(2,true);
           item->setRenameEnabled(3,true);
		   item->setRenameEnabled(4,true);
	}
}

